# this characters need escaping . ^ $ * + ? { } [ ] \ | ( )
import bslint.constants as const

List = [
    (r"\n", const.NEW_LINE, const.NO_INDENTATION),
    (r"\s+", None, const.NO_INDENTATION),

    ("\+=", const.OPERATOR, const.NO_INDENTATION),
    ("-=", const.OPERATOR, const.NO_INDENTATION),
    ("\*=", const.OPERATOR, const.NO_INDENTATION),
    ("/=", const.OPERATOR, const.NO_INDENTATION),
    (r"\\=", const.OPERATOR, const.NO_INDENTATION),  # divide integer
    ("<<=", const.OPERATOR, const.NO_INDENTATION),
    (">>=", const.OPERATOR, const.NO_INDENTATION),
    ("<>", const.OPERATOR, const.NO_INDENTATION),
    (">=", const.OPERATOR, const.NO_INDENTATION),
    ("<=", const.OPERATOR, const.NO_INDENTATION),
    (">=", const.OPERATOR, const.NO_INDENTATION),
    ("=>", const.OPERATOR, const.NO_INDENTATION),
    ("=<", const.OPERATOR, const.NO_INDENTATION),

    (">>", const.OPERATOR, const.NO_INDENTATION),
    ("<<", const.OPERATOR, const.NO_INDENTATION),

    (r"\.", const.SPECIAL_OPERATOR, const.NO_INDENTATION),
    (":", const.SPECIAL_OPERATOR, const.NO_INDENTATION),
    (r"\^", const.OPERATOR, const.NO_INDENTATION),
    ("=", const.OPERATOR, const.NO_INDENTATION),
    ("-", const.OPERATOR, const.NO_INDENTATION),
    ("\+", const.OPERATOR, const.NO_INDENTATION),
    ("\*", const.OPERATOR, const.NO_INDENTATION),
    (r"/", const.OPERATOR, const.NO_INDENTATION),
    ("<", const.OPERATOR, const.NO_INDENTATION),
    (">", const.OPERATOR, const.NO_INDENTATION),

    (r"\(", const.BRACKET, const.NO_INDENTATION),
    (r"\)", const.BRACKET, const.NO_INDENTATION),
    (r"\[", const.SQUARE_BRACKET, const.NO_INDENTATION),
    (r"\]", const.SQUARE_BRACKET, const.NO_INDENTATION),
    (r"\{", const.CURLY_BRACKET, const.NO_INDENTATION),
    (r"\}", const.CURLY_BRACKET, const.NO_INDENTATION),
    (",", const.SPECIAL_OPERATOR, const.NO_INDENTATION),
    (r"(TRUE)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(GETLASTRUNCOMPILEERROR)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(GETLASTRUNRUNTIMEERROR)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(GETGLOBALAA)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(ELSE IF)\b", const.KEYWORD, const.SPECIAL_INDENTATION),
    (r"(END IF)\b", const.KEYWORD, const.DECREMENT_INDENTATION),
    (r"(EXIT FOR)\b", const.KEYWORD, const.DECREMENT_INDENTATION),
    (r"(FOR EACH)\b", const.KEYWORD, const.INCREMENT_INDENTATION),
    (r"(END FOR)\b", const.KEYWORD, const.DECREMENT_INDENTATION),
    (r"(END WHILE)\b", const.KEYWORD, const.DECREMENT_INDENTATION),
    (r"(EXIT WHILE)\b", const.KEYWORD, const.DECREMENT_INDENTATION),
    (r"(END FUNCTION)\b", const.KEYWORD, const.DECREMENT_INDENTATION),
    (r"(END SUB)\b", const.KEYWORD, const.DECREMENT_INDENTATION),
    (r"(LINE_NUM)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(GOTO)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(CREATEOBJECT)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(ELSEIF)\b", const.KEYWORD, const.SPECIAL_INDENTATION),
    (r"(ENDIF)\b", const.KEYWORD, const.DECREMENT_INDENTATION),
    (r"(ENDWHILE)\b", const.KEYWORD, const.DECREMENT_INDENTATION),
    (r"(EXITWHILE)\b", const.KEYWORD, const.DECREMENT_INDENTATION),
    (r"(ENDFUNCTION)\b", const.KEYWORD, const.DECREMENT_INDENTATION),
    (r"(ENDSUB)\b", const.KEYWORD, const.DECREMENT_INDENTATION),
    (r"(OBJFUN)\b", const.KEYWORD, const.NO_INDENTATION),

    (r"(IF)\b", const.KEYWORD, const.INCREMENT_INDENTATION),
    (r"(THEN)\b", const.KEYWORD, const.INCREMENT_INDENTATION),
    (r"(ELSE)\b", const.KEYWORD, const.SPECIAL_INDENTATION),
    (r"(FOR)\b", const.KEYWORD, const.INCREMENT_INDENTATION),
    (r"(TO)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(STEP)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(INVALID)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(IN)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(WHILE)\b", const.KEYWORD, const.INCREMENT_INDENTATION),
    (r"(FUNCTION)\b", const.KEYWORD, const.INCREMENT_INDENTATION),
    (r"(AS)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(RETURN)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(PRINT)\b", const.PRINT_KEYWORD, const.NO_INDENTATION),
    (r"(DIM)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(END)\b", const.KEYWORD, const.DECREMENT_INDENTATION),
    (r"(MOD)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(STOP)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(AND)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(BOX)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(EACH)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(EVAL)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(EXIT)\b", const.KEYWORD, const.DECREMENT_INDENTATION),
    (r"(FALSE)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(LET)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(NEXT)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(NOT)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(OR)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(POS)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"('|rem) *(BSLINT)_(?P<command>[a-z_]+)(:(?P<param>[0-9]+))?.*", const.BSLINT_COMMAND, const.NO_INDENTATION),
    (r"('|REM)(.*)", const.COMMENT, const.NO_INDENTATION),
    (r"(RUN)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(SUB)\b", const.KEYWORD, const.INCREMENT_INDENTATION),
    (r"(TAB)\b", const.KEYWORD, const.NO_INDENTATION),
    (r"(TYPE)\b", const.KEYWORD, const.NO_INDENTATION),


    (r"(MAIN)\b", const.KEYWORD, const.NO_INDENTATION),
    ("\?", const.PRINT_KEYWORD, const.NO_INDENTATION),

    (r"(?P<value>^[a-z_][a-z0-9_]*)(?P<type>\$|%|!|#|&?)", const.ID, const.NO_INDENTATION),
    ('\"(.*)\"', const.STRING, const.NO_INDENTATION),
    (r"^\d*(\.?\d+){1}", const.NUMERIC, const.NO_INDENTATION)
]



import bslint.regexs as regexs
import re


def find_match(characters):
    for regex in regexs.List:
        match = re.match(regex[0], characters, re.IGNORECASE)
        if match:
            break
    if not match:
        raise ValueError('NO MATCH FOUND')
    return {"match": match, "token_type": regex[1], "indentation_level": regex[2]}



import bslint.error_messages_builder.error_builder.error_messages_constants as err


def get_message(key, params=[]):
    if key not in err.ERROR_TABLE:
        raise ValueError("No Error with such key")
    return err.ERROR_TABLE[key].get_message(params)






import re

import bslint.error_messages_builder.error_builder.error_messages_constants as err_const
import bslint.error_messages_builder.error_message_handler as err
import bslint.utilities.match_handler as match_handler
import bslint.utilities.regex_handler as regex_handler
from bslint import constants as const


def lex(characters):
    tokens = []
    errors = []
    handle_match = match_handler.MatchHandler(characters)
    while handle_match.current_char_index < len(characters):
        try:
            dividing_chars_in_tokens(characters, handle_match, tokens)
        except ValueError:
            handle_non_matching_token(characters, errors, handle_match)
    if handle_match._style_checking_is_active():
        handle_match._apply_new_line_styling()
    return build_return_dict(errors, handle_match, tokens)


def dividing_chars_in_tokens(characters, handle_match, tokens):
    result = regex_handler.find_match(characters[handle_match.current_char_index:])
    handle_match.line_length += len(result["match"].group())
    handle_match.current_char_index += len(result["match"].group())
    if result["token_type"] is not None:
        token_tuple = handle_match.match_handler(result)

        if token_tuple is not None:
            tokens.append(token_tuple)


def handle_non_matching_token(characters, errors, handle_match):
    end_of_line = re.match(r"(.*)\n", characters[handle_match.current_char_index:])
    errors.append(err.get_message(err_const.UNMATCHED_QUOTATION_MARK,
                                  [(end_of_line.group()[:const.PENULTIMATE_CHARACTER]),
                                   handle_match.line_number]))
    handle_match.line_number += 1
    handle_match.current_char_index += len(end_of_line.group())


def build_return_dict(errors, handle_match, tokens):
    if len(errors) is not 0:
        return {"Status": "Error", "Tokens": errors, "Warnings": handle_match.warnings}
    else:
        return {"Status": "Success", "Tokens": tokens, "Warnings": handle_match.warnings}




import bslint.lexer as lexer


def get_string_to_parse(filepath):
    fo = open(filepath, "r+")
    str_to_parse = fo.read()

    return str_to_parse


def lex_from_file():
    filepath = "../resources/SkeletonMain.brs"
    fo = open(filepath, "r+")
    str_to_parse = fo.read()
    print(lexer.lex(str_to_parse))

if __name__ == "__main__":
    lex_from_file()




import bslint.utilities.commands as commands


class FileReader:
    @staticmethod
    def read_file(file_to_lex):
        fo = open(file_to_lex, "r+")
        str_to_lex = fo.read()

        return {"invalid_encoding": commands.check_file_encoding(file_to_lex), "str_to_lex": str_to_lex}



import enchant
import os


def _get_new_dictionary(dictionary_lang="en_GB"):
    extra_words_filepath = _get_personal_words_filepath()
    return enchant.DictWithPWL(dictionary_lang, extra_words_filepath)


def _get_personal_words_filepath():
    this_dir, this_filename = os.path.split(__file__)
    file_path = os.path.join(this_dir, "../config/personal-words-list.txt")
    return file_path


import bslint.error_messages_builder.error_builder.error_message_builder as ErrBuilder

ERROR = "ERROR"
WARNING = "WARNING"

LINE_LENGTH = "LINE_LENGTH"
FILE_ENCODING = "FILE_ENCODING"
CONSECUTIVE_EMPTY_LINES = "CONSECUTIVE_EMPTY_LINES"
UPPERCASE_VARIABLE = "UPPERCASE_VARIABLE"
TAB_INDENTATION_ERROR = "TAB_INDENTATION_ERROR"
TAB_AND_SPACES = "TAB_AND_SPACES"
TYPO_IN_CODE = "TYPO_IN_CODE"
TRACEABLE_CODE = "TRACEABLE_CODE"
NON_CONVENTIONAL_TODO = "NON_CONVENTIONAL_TODO"
COMMENTS_NOT_ALLOWED = "COMMENTS_NOT_ALLOWED"
NO_RETURN_AND_PARAMETERS_TYPES = "NO_RETURN_AND_PARAMETERS_TYPES"
NOT_CAPITALISED_VARIABLE_NAMES = "NOT_CAPITALISED_VARIABLE_NAMES"
NO_SPACE_AROUND_OPERATORS = "NO_SPACE_AROUND_OPERATORS = {NO_SPACE_AROUND_OPERATORS"
NO_SPACE_BETWEEN_METHOD_AND_PARAMETERS = "NO_SPACE_BETWEEN_METHOD_AND_PARAMETERS"
UNMATCHED_QUOTATION_MARK = "UNMATCHED_QUOTATION_MARK"
NON_CONVENTIONAL_TODO_AND_NO_COMMENTS = "NON_CONVENTIONAL_TODO_AND_NO_COMMENTS"
NO_TODOS = "NO_TODOS"
METHOD_DECLARATION_SPACING = "METHOD_DECLARATION_SPACING"

ERROR_TABLE = {
        LINE_LENGTH: ErrBuilder.Error(WARNING + ": Line length exceeds {} number of characters. Line number: {}"),
        FILE_ENCODING: ErrBuilder.Error(WARNING + ": Your files should conform to {} encoding"),
        CONSECUTIVE_EMPTY_LINES: ErrBuilder.Error(WARNING + ": Cannot have more than {} consecutive empty lines. Line number: {}"),
        UPPERCASE_VARIABLE: ErrBuilder.Error(WARNING + ": Variables should begin with a lowercase letter: line number: {}"),
        TAB_INDENTATION_ERROR: ErrBuilder.Error(WARNING + ": Tab indentation should be set to {} spaces. Line number: {}"),
        TAB_AND_SPACES: ErrBuilder.Error(WARNING + ": Invalid indentation, you must indent with tabs. Line number: {}"),
        TYPO_IN_CODE: ErrBuilder.Error(WARNING + ": You have spelling mistakes in your code. Line number: {}"),
        TRACEABLE_CODE: ErrBuilder.Error(WARNING + ": Print statements not allowed. Line number: {}"),
        NON_CONVENTIONAL_TODO_AND_NO_COMMENTS: ErrBuilder.Error(WARNING + ": Comments must be TODOs and must follow convention. Line number: {}"),
        COMMENTS_NOT_ALLOWED: ErrBuilder.Error(WARNING + ": No comments allowed. Line number: {}"),
        NO_RETURN_AND_PARAMETERS_TYPES: ErrBuilder.Error(WARNING + ": No return and parameter types declared. Line number: {}"),
        NOT_CAPITALISED_VARIABLE_NAMES: ErrBuilder.Error(WARNING + ": Variable types must be capitalised. Line number: {}"),
        NO_SPACE_AROUND_OPERATORS: ErrBuilder.Error(WARNING + ": Operators must be surrounded by {} space(s). Line number: {}"),
        NO_SPACE_BETWEEN_METHOD_AND_PARAMETERS: ErrBuilder.Error( WARNING + ": You must put a space between method name and parameter list. Line number: {}"),
        UNMATCHED_QUOTATION_MARK: ErrBuilder.Error(ERROR + ": You have unmatched quotation marks at {} on line number: {}"),
        NON_CONVENTIONAL_TODO: ErrBuilder.Error(WARNING + ": TODOs must follow convention. Line number: {}"),
        NO_TODOS: ErrBuilder.Error(WARNING + ": Comments must not be TODOs. Line number: {}"),
        METHOD_DECLARATION_SPACING: ErrBuilder.Error(WARNING + ": The spacing on the method declaration is incorrect: {}"),
    }



STMT = 'STMT'
ID = 'ID'
STRING = 'STRING'
INT = 'INT'
NUMERIC = 'NUMERIC'
NEW_LINE = "NEW_LINE"
OPERATOR = "OPERATOR"
SPECIAL_OPERATOR = "SPECIAL_OPERATOR"
BRACKET = "BRACKET"
SQUARE_BRACKET = "SQUARE_BRACKET"
KEYWORD = "KEYWORD"
PRINT_KEYWORD = "PRINT_KEYWORD"
BSLINT_COMMAND = 'BSLINT_COMMAND'
COMMENT = "COMMENT"
ERROR = "ERROR"
WARNING = "WARNING"
INCREMENT_INDENTATION = 1
DECREMENT_INDENTATION = -1
NO_INDENTATION = 0
SPECIAL_INDENTATION = 2
WARNING_COLOUR = '\033[93m'
PASS_COLOUR = '\033[92m'
END_COLOUR = '\033[0m'
ERROR_COLOUR = '\033[91m'
FILE_COLOUR = '\033[34;4m'
TITLE_COLOUR = '\033[95m'
HEADER_COLOUR = '\033[34;4m'
PENULTIMATE_CHARACTER = -2
CURLY_BRACKET = 'CURLY_BRACKET'



# -*- coding: utf-8 -*-
"""
    BSLint
    ~~~~~
    A linter for the BrightScript language.
    :license: BSD, see LICENSE for more details.
"""

__version__ = '0.4.15'

from .utilities.commands import *
from .utilities.config_loader import *
from .utilities.file_reader import *
from .utilities.regex_handler import *
from .bslint import *
from .lexer import lex
from .main import get_string_to_parse




import codecs
import re

import bslint.error_messages_builder.error_builder.error_messages_constants as err_const
import bslint.utilities.config_loader as config_loader
import bslint.utilities.words_dictionary as words_dict
import bslint.constants as const

config = config_loader.CONFIG
dictionary = words_dict._get_new_dictionary()


def check_comment(token):
    if _command_is_active("check_comment") is not True:
        return

    check_comment_config = config["check_comment"]["params"]
    allow_todos = check_comment_config["TODOs"]["allow_TODOs"]
    allow_generic_comments = check_comment_config["allow_generic_comments"]
    todos_format = check_comment_config["TODOs"]["format"]

    if allow_todos and allow_generic_comments:
        if re.match(r"('|rem)\s*TODO", token):
            if not re.match(todos_format, token):
                return {"error_key": err_const.NON_CONVENTIONAL_TODO, "error_params": []}

    elif allow_todos and not allow_generic_comments:
        if not re.match(todos_format, token):
            return {"error_key": err_const.NON_CONVENTIONAL_TODO_AND_NO_COMMENTS, "error_params": []}

    elif not allow_todos and allow_generic_comments:
        if re.match(r"('|rem)\s*TODO", token):
            return {"error_key": err_const.NO_TODOS, "error_params": []}

    else:
        return {"error_key": err_const.COMMENTS_NOT_ALLOWED, "error_params": []}
    return


def check_file_encoding(file_path):
    if _command_is_active("check_file_encoding") is not True:
        return

    file_encoding = config['check_file_encoding']['params']
    try:
        codecs.open(file_path, encoding=file_encoding["source_file_encoding"]).read()
    except:
        return {"error_key": err_const.FILE_ENCODING, "error_params": []}


def check_indentation(current_indentation_level, characters, indentation_level):
    if indentation_level == const.DECREMENT_INDENTATION or \
                    indentation_level == const.SPECIAL_INDENTATION:
        current_indentation_level -= 1
    warning = _handle_warnings(current_indentation_level, characters)
    if indentation_level == const.INCREMENT_INDENTATION or \
                    indentation_level == const.SPECIAL_INDENTATION:
        current_indentation_level += 1

    return warning, current_indentation_level


def check_trace_free():
    if _command_is_active("check_trace_free") is True:
        return {"error_key": err_const.TRACEABLE_CODE, "error_params": []}


def check_max_line_length(line_length):
    if _command_is_active("max_line_length") is not True:
        return

    max_len = config['max_line_length']['params']['max_line_length']
    if max_len < line_length:
        return {"error_key": err_const.LINE_LENGTH, "error_params": [max_len]}


def check_consecutive_empty_lines(empty_lines):
    if _command_is_active("consecutive_empty_lines") is not True:
        return

    params = config["consecutive_empty_lines"]["params"]
    empty_lines_allowed = params["consecutive_empty_lines"]
    if empty_lines > empty_lines_allowed:
        return {"error_key": err_const.CONSECUTIVE_EMPTY_LINES, "error_params": [empty_lines_allowed]}


def check_skip_file():
    if _command_is_active("skip_file") is not True:
        return

    return True


def check_skip_line(line_number):
    if _command_is_active("skip_line") is not True:
        return

    return line_number + 1


def check_spaces_around_operators(characters, current_char_index):
    if _command_is_active("spaces_around_operators") is not True:
        return

    allowed_num_spaces = config["spaces_around_operators"]["params"]["spaces_around_operators"]
    before_index = current_char_index - allowed_num_spaces - 2
    after_index = current_char_index + allowed_num_spaces + 1
    chars_around_operator = characters[before_index:after_index]
    if not re.match("(\S{0,1})\s{" + str(allowed_num_spaces) + "}\S\s{" + str(
            allowed_num_spaces) + "}\S{0,1}$", chars_around_operator):
        return {"error_key": err_const.NO_SPACE_AROUND_OPERATORS,
                "error_params": [allowed_num_spaces]}


def check_spelling(token, token_type):
    if _command_is_active("spell_check") is not True:
        return

    if token_type == const.COMMENT:
        words = _parse_comment_words(token)
    else:
        words = _parse_words(token)

    for word in words:
        spelt_correct = dictionary.check(word)
        if not spelt_correct:
            return {"error_key": err_const.TYPO_IN_CODE, "error_params": []}


def check_method_declaration_spacing(token):
    if _command_is_active("check_method_declaration_spacing") is not True:
        return

    method_spaces = config["check_method_declaration_spacing"]["params"]["method_spaces"]

    if re.search(r"\b(function|sub)\b", token):
        if not re.search(r"(function|sub)\s{" + str(method_spaces) + "}[a-z0-9_A-Z]*\(([a-z0-9_A-Z]*(?:,\s{" + \
                                 str(method_spaces) + "}[a-z0-9_A-Z]*)?)*\)", token):
            return {"error_key": err_const.METHOD_DECLARATION_SPACING, "error_params": []}

# region Private helper functions


def _command_is_active(command_name):
    if command_name in config:
        return config[command_name]["active"]
    else:
        return False


def _change_dict_lang(dict_lang):
    global dictionary
    dictionary = words_dict._get_new_dictionary(dict_lang)


def _handle_warnings(current_indentation_level, characters):
    indent_config = config['check_indentation']['params']
    only_tab_indents = indent_config["only_tab_indents"]
    tab_size = indent_config["tab_size"]
    if re.search(r"\S", characters):
        if only_tab_indents:
            if not re.match("\t{" + str(current_indentation_level) + "}\S", characters):
                return {"error_key": err_const.TAB_AND_SPACES, "error_params": []}
        else:
            if not re.match("\s{" + str(
                            tab_size *
                            current_indentation_level) + "}\S",
                            characters):
                return {"error_key": err_const.TAB_INDENTATION_ERROR,
                        "error_params": [tab_size]}


def _parse_words(identifier_str):
    words = []
    word = ''

    for i in range(0, len(identifier_str)):
        char = identifier_str[i]
        if not char.isalpha():
            if not word == '':
                words.append(word)
            word = ''
            continue
        if char.islower():
            word += char
        elif char.isupper():
            if not word == '':
                words.append(word)
            word = ''
            word += char
        elif char == '_' and word != '':
            words.append(word)
            word = ''
    if not word == '':
        words.append(word)

    return words


def _parse_comment_words(comment):
    words = []
    comment_list = comment.split(' ')

    for word in comment_list:
        if not word.isalpha():
            continue
        if word == 'rem':
            continue
        if word.isupper():
            continue
        else:
            if not word == '':
                words.append(word)
    return words
# endregion



"""bslint.bslint: provides entry point main()."""
__version__ = "0.5.0"

import os
import sys

import bslint
import bslint.constants as const
import bslint.lexer as lexer

is_lexed_correctly = True
files = []


def main():
    print(const.TITLE_COLOUR + "BSLint: A linter for BrightScript. %s. \n" % __version__ + const.END_COLOUR)
    global is_lexed_correctly
    try:
        manifest_path = find_manifest()
    except FileNotFoundError:
        manifest_path = ""
        print(const.ERROR_COLOUR + "No manifest file found" + const.END_COLOUR)

    try:
        bslintrc_path = os.path.join(manifest_path, ".bslintrc")
        bslintrc = read_bslintrc(bslintrc_path)
        bslint.config_loader.load_config_file(bslintrc_path)
    except FileNotFoundError:
        print(const.ERROR_COLOUR + "No .bslintrc file found" + const.END_COLOUR)
        bslintrc = {"ignore": ""}

    if is_not_specific_path():
        pathname = os.getcwd()
        lint_all(pathname, bslintrc["ignore"], manifest_path)
    else:
        filename = sys.argv[1]
        if not os.path.exists(filename):
            is_lexed_correctly = False

        if os.path.isfile(filename):
            lint_specific(filename)
        else:
            lint_all(filename, bslintrc["ignore"], manifest_path)
    if is_lexed_correctly:
        print(const.PASS_COLOUR + 'All lexed correctly' + const.END_COLOUR)
        print("\n")


def lint_all(directory, directories_to_ignore, manifest_path):
    for dirName, subdirList, files in os.walk(directory):
        if not ignore_dir(get_relative_path(dirName, manifest_path), directories_to_ignore):
            for file in files:
                if file.endswith(".brs") or file.endswith(".bs"):
                    filepath = os.path.join(dirName, file)
                    lint_specific(filepath)


def get_relative_path(dirName, directory):
    directory = os.path.realpath(os.path.join(os.getcwd(), directory))
    return dirName.replace(directory, '')


def ignore_dir(relative_directory_path, directories_to_ignore):
    for directory in directories_to_ignore:
        directory = add_slashes_to_dir(directory)
        relative_directory_path = add_slashes_to_dir(relative_directory_path)

        if relative_directory_path.startswith(directory):
            return True
    return False


def add_slashes_to_dir(directory):
    if not directory.startswith("/"):
        directory = "/" + directory
    if not directory.endswith("/"):
        directory += "/"
    return directory


def lint_specific(filename):
    filename = filename.replace(os.getcwd() + '/', '')
    if filename.endswith(".brs") or filename.endswith(".bs"):
        files.append(filename)
        file_reader = bslint.FileReader()
        read_file = file_reader.read_file(filename)
        if read_file["invalid_encoding"]:
            print(read_file["invalid_encoding"])
        lex_result = lexer.lex(read_file['str_to_lex'])
        if lex_result["Status"] == "Error" or lex_result["Warnings"]:
            global is_lexed_correctly
            is_lexed_correctly = False
            print(const.FILE_COLOUR + filename + const.END_COLOUR)
            if lex_result["Status"] == "Error":
                for error in lex_result["Tokens"]:
                    print(const.ERROR_COLOUR + str(error) + const.END_COLOUR)
            for warning in lex_result["Warnings"]:
                print(const.WARNING_COLOUR + str(warning) + const.END_COLOUR)
            print("\n")
    else:
        is_lexed_correctly = False


def find_manifest():
    if is_not_specific_path():
        upper_dir = ""
    else:
        if os.path.isfile(sys.argv[1]):
            upper_dir = os.path.dirname(sys.argv[1])
        else:
            upper_dir = sys.argv[1]
    count = 0
    while no_manifest_in_folder(upper_dir) and count < 5:
        upper_dir = os.path.join(upper_dir, "../")
        count += 1
    if count == 5:
        raise FileNotFoundError
    return upper_dir


def no_manifest_in_folder(upper_dir):
    return not os.path.exists(os.path.join(upper_dir, "MANIFEST")) and not os.path.exists(
        os.path.join(upper_dir, "manifest"))


def read_bslintrc(manifest_dir):
    if manifest_dir is not None:
        bslintrc = bslint.config_loader.read_json(manifest_dir)
    else:
        raise FileNotFoundError
    return bslintrc


def runner(to_lex=None):
    global files
    files = []
    sys.argv = [sys.argv[0]]
    if to_lex is not None:
        sys.argv.append(os.path.join(os.getcwd(), to_lex))
    else:
        sys.argv.append(os.getcwd())
    main()
    return files


def is_not_specific_path():
    return len(sys.argv) == 1


import json
import sys
import os


def load_config_file(user=None, default=None, out=sys.stdout):
    this_dir, this_filename = os.path.split(__file__)
    tests_filepath_prefix = os.path.join(this_dir, "../../tests/resources/config/")
    filepath_prefix = os.path.join(this_dir, "../../bslint/config/")

    user_filepath = get_user_config(filepath_prefix, tests_filepath_prefix, user)
    if default:
        default_filepath = tests_filepath_prefix + default
    else:
        default_filepath = filepath_prefix + "default-config.json"

    try:
        default_json = read_json(default_filepath)
        user_json = read_json(user_filepath)
        for property in user_json:
            default_json[property] = user_json[property]
    except FileNotFoundError as e:
        out.write("Cannot find file: " + e.filename)
    else:
        global CONFIG
        CONFIG = default_json
        return default_json


def get_user_config(filepath_prefix, tests_filepath_prefix, user):
    user_filepath = user
    if user is None:
        user_filepath = filepath_prefix + "user-config.json"
    elif not os.path.isfile(user):
        user_filepath = tests_filepath_prefix + user
    return user_filepath


def read_json(filepath):
    config_string = ''
    with open(filepath) as f:
        for line in f:
            line = line.lstrip()
            if not line.startswith("//"):
                config_string += line
    config_json = json.loads(config_string)
    return config_json


CONFIG = load_config_file()


if __name__ == "__main__":
    load_config_file()